@@ -27,8 +27,15 @@
 #include "backends/meta-monitor-manager-private.h"
 #include "backends/meta-settings-private.h"
 
+#define SCALE_FACTORS_PER_INTEGER 8
+#define MINIMUM_SCALE_FACTOR 0.5f
+#define MAXIMUM_SCALE_FACTOR 4.0f
+
+#define HANDLED_CRTC_MODE_FLAGS (META_CRTC_MODE_FLAG_INTERLACE)
+
 typedef struct _MetaMonitorMode
 {
+  char *id;
   MetaMonitorModeSpec spec;
   MetaMonitorCrtcMode *crtc_modes;
 } MetaMonitorMode;
@@ -46,6 +53,7 @@ typedef struct _MetaMonitorPrivate
 
   GList *outputs;
   GList *modes;
+  GHashTable *mode_ids;
 
   MetaMonitorMode *preferred_mode;
   MetaMonitorMode *current_mode;
@@ -250,11 +258,10 @@ meta_monitor_get_current_resolution (MetaMonitor *monitor,
 }
 
 void
-meta_monitor_derive_dimensions (MetaMonitor   *monitor,
-                                int           *width,
-                                int           *height)
+meta_monitor_derive_layout (MetaMonitor   *monitor,
+                            MetaRectangle *layout)
 {
-  META_MONITOR_GET_CLASS (monitor)->derive_dimensions (monitor, width, height);
+  META_MONITOR_GET_CLASS (monitor)->derive_layout (monitor, layout);
 }
 
 void
@@ -329,6 +336,7 @@ meta_monitor_finalize (GObject *object)
   MetaMonitor *monitor = META_MONITOR (object);
   MetaMonitorPrivate *priv = meta_monitor_get_instance_private (monitor);
 
+  g_hash_table_destroy (priv->mode_ids);
   g_list_free_full (priv->modes, (GDestroyNotify) meta_monitor_mode_free);
   g_clear_pointer (&priv->outputs, g_list_free);
   meta_monitor_spec_free (priv->spec);
@@ -337,6 +345,9 @@ meta_monitor_finalize (GObject *object)
 static void
 meta_monitor_init (MetaMonitor *monitor)
 {
+  MetaMonitorPrivate *priv = meta_monitor_get_instance_private (monitor);
+
+  priv->mode_ids = g_hash_table_new (g_str_hash, g_str_equal);
 }
 
 static void
@@ -347,6 +358,39 @@ meta_monitor_class_init (MetaMonitorClass *klass)
   object_class->finalize = meta_monitor_finalize;
 }
 
+static char *
+generate_mode_id (MetaMonitorModeSpec *monitor_mode_spec)
+{
+  gboolean is_interlaced;
+  char refresh_rate_str[G_ASCII_DTOSTR_BUF_SIZE];
+
+  is_interlaced = !!(monitor_mode_spec->flags & META_CRTC_MODE_FLAG_INTERLACE);
+  g_ascii_dtostr (refresh_rate_str, G_ASCII_DTOSTR_BUF_SIZE,
+                  monitor_mode_spec->refresh_rate);
+
+  return g_strdup_printf ("%dx%d%s@%s",
+                          monitor_mode_spec->width,
+                          monitor_mode_spec->height,
+                          is_interlaced ? "i" : "",
+                          refresh_rate_str);
+}
+
+static gboolean
+meta_monitor_add_mode (MetaMonitor     *monitor,
+                       MetaMonitorMode *monitor_mode)
+{
+  MetaMonitorPrivate *priv = meta_monitor_get_instance_private (monitor);
+
+  if (g_hash_table_lookup (priv->mode_ids,
+                           meta_monitor_mode_get_id (monitor_mode)))
+    return FALSE;
+
+  priv->modes = g_list_append (priv->modes, monitor_mode);
+  g_hash_table_insert (priv->mode_ids, monitor_mode->id, monitor_mode);
+
+  return TRUE;
+}
+
 static void
 meta_monitor_normal_generate_modes (MetaMonitorNormal *monitor_normal)
 {
@@ -366,8 +410,10 @@ meta_monitor_normal_generate_modes (MetaMonitorNormal *monitor_normal)
       mode->spec = (MetaMonitorModeSpec) {
         .width = crtc_mode->width,
         .height = crtc_mode->height,
-        .refresh_rate = crtc_mode->refresh_rate
+        .refresh_rate = crtc_mode->refresh_rate,
+        .flags = crtc_mode->flags & HANDLED_CRTC_MODE_FLAGS
       },
+      mode->id = generate_mode_id (&mode->spec);
       mode->crtc_modes = g_new (MetaMonitorCrtcMode, 1);
       mode->crtc_modes[0] = (MetaMonitorCrtcMode) {
         .output = output,
@@ -379,7 +425,8 @@ meta_monitor_normal_generate_modes (MetaMonitorNormal *monitor_normal)
       if (output->crtc && crtc_mode == output->crtc->current_mode)
         monitor_priv->current_mode = mode;
 
-      monitor_priv->modes = g_list_append (monitor_priv->modes, mode);
+      if (!meta_monitor_add_mode (monitor, mode))
+        meta_monitor_mode_free (mode);
     }
 }
 
@@ -416,15 +463,18 @@ meta_monitor_normal_get_main_output (MetaMonitor *monitor)
 }
 
 static void
-meta_monitor_normal_derive_dimensions (MetaMonitor   *monitor,
-                                       int           *width,
-                                       int           *height)
+meta_monitor_normal_derive_layout (MetaMonitor   *monitor,
+                                   MetaRectangle *layout)
 {
   MetaOutput *output;
 
   output = meta_monitor_get_main_output (monitor);
-  *width = output->crtc->rect.width;
-  *height = output->crtc->rect.height;
+  *layout = (MetaRectangle) {
+    .x = output->crtc->rect.x,
+    .y = output->crtc->rect.y,
+    .width = output->crtc->rect.width,
+    .height = output->crtc->rect.height
+  };
 }
 
 static gboolean
@@ -467,7 +517,7 @@ meta_monitor_normal_class_init (MetaMonitorNormalClass *klass)
   MetaMonitorClass *monitor_class = META_MONITOR_CLASS (klass);
 
   monitor_class->get_main_output = meta_monitor_normal_get_main_output;
-  monitor_class->derive_dimensions = meta_monitor_normal_derive_dimensions;
+  monitor_class->derive_layout = meta_monitor_normal_derive_layout;
   monitor_class->calculate_crtc_pos = meta_monitor_normal_calculate_crtc_pos;
   monitor_class->get_suggested_position = meta_monitor_normal_get_suggested_position;
 }
@@ -632,8 +682,8 @@ is_crtc_mode_tiled (MetaOutput   *output,
 }
 
 static MetaCrtcMode *
-find_tiled_crtc_mode (MetaOutput *output,
-                      float       refresh_rate)
+find_tiled_crtc_mode (MetaOutput   *output,
+                      MetaCrtcMode *reference_crtc_mode)
 {
   MetaCrtcMode *crtc_mode;
   unsigned int i;
@@ -649,7 +699,10 @@ find_tiled_crtc_mode (MetaOutput *output,
       if (!is_crtc_mode_tiled (output, crtc_mode))
         continue;
 
-      if (crtc_mode->refresh_rate != refresh_rate)
+      if (crtc_mode->refresh_rate != reference_crtc_mode->refresh_rate)
+        continue;
+
+      if (crtc_mode->flags != reference_crtc_mode->flags)
         continue;
 
       return crtc_mode;
@@ -660,22 +713,30 @@ find_tiled_crtc_mode (MetaOutput *output,
 
 static MetaMonitorMode *
 create_tiled_monitor_mode (MetaMonitorTiled *monitor_tiled,
-                           float             refresh_rate,
+                           MetaCrtcMode     *reference_crtc_mode,
                            gboolean         *out_is_preferred)
 {
   MetaMonitor *monitor = META_MONITOR (monitor_tiled);
   MetaMonitorPrivate *monitor_priv =
     meta_monitor_get_instance_private (monitor);
   MetaMonitorModeTiled *mode;
+  int width, height;
   GList *l;
   unsigned int i;
   gboolean is_preferred = TRUE;
 
   mode = g_new0 (MetaMonitorModeTiled, 1);
   mode->is_tiled = TRUE;
-  meta_monitor_tiled_calculate_tiled_size (monitor,
-                                           &mode->parent.spec.width,
-                                           &mode->parent.spec.height);
+  meta_monitor_tiled_calculate_tiled_size (monitor, &width, &height);
+
+  mode->parent.spec = (MetaMonitorModeSpec) {
+    .width = width,
+    .height = height,
+    .refresh_rate = reference_crtc_mode->refresh_rate,
+    .flags = reference_crtc_mode->flags & HANDLED_CRTC_MODE_FLAGS
+  };
+  mode->parent.id = generate_mode_id (&mode->parent.spec);
+
   mode->parent.crtc_modes = g_new0 (MetaMonitorCrtcMode,
                                     g_list_length (monitor_priv->outputs));
   for (l = monitor_priv->outputs, i = 0; l; l = l->next, i++)
@@ -683,11 +744,10 @@ create_tiled_monitor_mode (MetaMonitorTiled *monitor_tiled,
       MetaOutput *output = l->data;
       MetaCrtcMode *tiled_crtc_mode;
 
-      tiled_crtc_mode = find_tiled_crtc_mode (output, refresh_rate);
+      tiled_crtc_mode = find_tiled_crtc_mode (output, reference_crtc_mode);
       if (!tiled_crtc_mode)
         {
-          g_warning ("No tiled mode with refresh rate %f on %s",
-                     refresh_rate, output->name);
+          g_warning ("No tiled mode found on %s", output->name);
           meta_monitor_mode_free ((MetaMonitorMode *) mode);
           return NULL;
         }
@@ -697,8 +757,6 @@ create_tiled_monitor_mode (MetaMonitorTiled *monitor_tiled,
         .crtc_mode = tiled_crtc_mode
       };
 
-      mode->parent.spec.refresh_rate = refresh_rate;
-
       is_preferred = is_preferred && tiled_crtc_mode == output->preferred_mode;
     }
 
@@ -716,19 +774,21 @@ generate_tiled_monitor_modes (MetaMonitorTiled *monitor_tiled)
   MetaOutput *main_output;
   GList *tiled_modes = NULL;
   unsigned int i;
+  MetaMonitorMode *best_mode = NULL;
+  GList *l;
 
   main_output = meta_monitor_get_main_output (META_MONITOR (monitor_tiled));
 
   for (i = 0; i < main_output->n_modes; i++)
     {
-      MetaCrtcMode *crtc_mode = main_output->modes[i];
+      MetaCrtcMode *reference_crtc_mode = main_output->modes[i];
       MetaMonitorMode *mode;
       gboolean is_preferred;
 
-      if (!is_crtc_mode_tiled (main_output, crtc_mode))
+      if (!is_crtc_mode_tiled (main_output, reference_crtc_mode))
         continue;
 
-      mode = create_tiled_monitor_mode (monitor_tiled, crtc_mode->refresh_rate,
+      mode = create_tiled_monitor_mode (monitor_tiled, reference_crtc_mode,
                                         &is_preferred);
       if (!mode)
         continue;
@@ -742,24 +802,28 @@ generate_tiled_monitor_modes (MetaMonitorTiled *monitor_tiled)
         monitor_priv->preferred_mode = mode;
     }
 
-  if (!monitor_priv->preferred_mode)
+  while ((l = tiled_modes))
     {
-      MetaMonitorMode *best_mode = NULL;
-      GList *l;
+      MetaMonitorMode *mode = l->data;
 
-      for (l = tiled_modes; l; l = l->next)
+      tiled_modes = g_list_remove_link (tiled_modes, l);
+
+      if (!meta_monitor_add_mode (monitor, mode))
         {
-          MetaMonitorMode *mode = l->data;
+          meta_monitor_mode_free (mode);
+          continue;
+        }
 
+      if (!monitor_priv->preferred_mode)
+        {
           if (!best_mode ||
               mode->spec.refresh_rate > best_mode->spec.refresh_rate)
             best_mode = mode;
         }
-
-      monitor_priv->preferred_mode = best_mode;
     }
 
-  monitor_priv->modes = g_list_concat (monitor_priv->modes, tiled_modes);
+  if (best_mode)
+    monitor_priv->preferred_mode = best_mode;
 }
 
 static MetaMonitorMode *
@@ -783,10 +847,12 @@ create_untiled_monitor_mode (MetaMonitorTiled *monitor_tiled,
   mode->parent.spec = (MetaMonitorModeSpec) {
     .width = crtc_mode->width,
     .height = crtc_mode->height,
-    .refresh_rate = crtc_mode->refresh_rate
+    .refresh_rate = crtc_mode->refresh_rate,
+    .flags = crtc_mode->flags & HANDLED_CRTC_MODE_FLAGS
   };
+  mode->parent.id = generate_mode_id (&mode->parent.spec);
   mode->parent.crtc_modes = g_new0 (MetaMonitorCrtcMode,
-                             g_list_length (monitor_priv->outputs));
+                                    g_list_length (monitor_priv->outputs));
 
   for (l = monitor_priv->outputs, i = 0; l; l = l->next, i++)
     {
@@ -884,7 +950,11 @@ generate_untiled_monitor_modes (MetaMonitorTiled *monitor_tiled)
       if (!mode)
         continue;
 
-      monitor_priv->modes = g_list_append (monitor_priv->modes, mode);
+      if (!meta_monitor_add_mode (monitor, mode))
+        {
+          meta_monitor_mode_free (mode);
+          continue;
+        }
 
       if (is_monitor_mode_assigned (monitor, mode))
         {
@@ -1033,9 +1103,8 @@ meta_monitor_tiled_get_main_output (MetaMonitor *monitor)
 }
 
 static void
-meta_monitor_tiled_derive_dimensions (MetaMonitor   *monitor,
-                                      int           *out_width,
-                                      int           *out_height)
+meta_monitor_derived_derive_layout (MetaMonitor   *monitor,
+                                    MetaRectangle *layout)
 {
   MetaMonitorPrivate *monitor_priv =
     meta_monitor_get_instance_private (monitor);
@@ -1059,8 +1128,12 @@ meta_monitor_tiled_derive_dimensions (MetaMonitor   *monitor,
       max_y = MAX (output->crtc->rect.y + output->crtc->rect.height, max_y);
     }
 
-  *out_width = max_x - min_x;
-  *out_height = max_y - min_y;
+  *layout = (MetaRectangle) {
+    .x = min_x,
+    .y = min_y,
+    .width = max_x - min_x,
+    .height = max_y - min_y
+  };
 }
 
 static gboolean
@@ -1118,7 +1191,7 @@ meta_monitor_tiled_class_init (MetaMonitorTiledClass *klass)
   object_class->finalize = meta_monitor_tiled_finalize;
 
   monitor_class->get_main_output = meta_monitor_tiled_get_main_output;
-  monitor_class->derive_dimensions = meta_monitor_tiled_derive_dimensions;
+  monitor_class->derive_layout = meta_monitor_derived_derive_layout;
   monitor_class->calculate_crtc_pos = meta_monitor_tiled_calculate_crtc_pos;
   monitor_class->get_suggested_position = meta_monitor_tiled_get_suggested_position;
 }
@@ -1126,6 +1199,7 @@ meta_monitor_tiled_class_init (MetaMonitorTiledClass *klass)
 static void
 meta_monitor_mode_free (MetaMonitorMode *monitor_mode)
 {
+  g_free (monitor_mode->id);
   g_free (monitor_mode->crtc_modes);
   g_free (monitor_mode);
 }
@@ -1149,6 +1223,15 @@ meta_monitor_get_logical_monitor (MetaMonitor *monitor)
     return NULL;
 }
 
+MetaMonitorMode *
+meta_monitor_get_mode_from_id (MetaMonitor *monitor,
+                               const char  *monitor_mode_id)
+{
+  MetaMonitorPrivate *priv = meta_monitor_get_instance_private (monitor);
+
+  return g_hash_table_lookup (priv->mode_ids, monitor_mode_id);
+}
+
 static gboolean
 meta_monitor_mode_spec_equals (MetaMonitorModeSpec *monitor_mode_spec,
                                MetaMonitorModeSpec *other_monitor_mode_spec)
@@ -1156,7 +1239,8 @@ meta_monitor_mode_spec_equals (MetaMonitorModeSpec *monitor_mode_spec,
   return (monitor_mode_spec->width == other_monitor_mode_spec->width &&
           monitor_mode_spec->height == other_monitor_mode_spec->height &&
           (monitor_mode_spec->refresh_rate ==
-           other_monitor_mode_spec->refresh_rate));
+           other_monitor_mode_spec->refresh_rate) &&
+          monitor_mode_spec->flags == other_monitor_mode_spec->flags);
 }
 
 MetaMonitorMode *
@@ -1261,7 +1345,7 @@ meta_monitor_calculate_crtc_pos (MetaMonitor         *monitor,
 /* From http://en.wikipedia.org/wiki/4K_resolution#Resolutions_of_common_formats */
 #define SMALLEST_4K_WIDTH 3656
 
-static int
+static float
 calculate_scale (MetaMonitor     *monitor,
                  MetaMonitorMode *monitor_mode)
 {
@@ -1269,7 +1353,7 @@ calculate_scale (MetaMonitor     *monitor,
   int width_mm, height_mm;
   int scale;
 
-  scale = 1;
+  scale = 1.0;
 
   meta_monitor_mode_get_resolution (monitor_mode,
                                     &resolution_width,
@@ -1314,14 +1398,14 @@ calculate_scale (MetaMonitor     *monitor,
        * pick higher ratio than 2 automatically.
        */
       if (dpi_x > HIDPI_LIMIT && dpi_y > HIDPI_LIMIT)
-        scale = 2;
+        scale = 2.0;
     }
 
 out:
   return scale;
 }
 
-int
+float
 meta_monitor_calculate_mode_scale (MetaMonitor     *monitor,
                                    MetaMonitorMode *monitor_mode)
 {
@@ -1336,12 +1420,130 @@ meta_monitor_calculate_mode_scale (MetaMonitor     *monitor,
   return calculate_scale (monitor, monitor_mode);
 }
 
+static float
+get_closest_scale_factor_for_resolution (float width,
+                                         float height,
+                                         float scale,
+                                         float scale_step)
+{
+  unsigned int i, j;
+  float scaled_h;
+  float scaled_w;
+  float best_scale;
+  int base_scaled_w;
+  gboolean limit_exceeded;
+  gboolean found_one;
+
+  best_scale = 0;
+  scaled_w = width / scale;
+  scaled_h = height / scale;
+
+  if (scale < MINIMUM_SCALE_FACTOR || scale > MAXIMUM_SCALE_FACTOR)
+    goto out;
+
+  if (floorf (scaled_w) == scaled_w && floorf (scaled_h) == scaled_h)
+    return scale;
+
+  i = 0;
+  found_one = FALSE;
+  limit_exceeded = FALSE;
+  base_scaled_w = floorf (scaled_w);
+
+  do
+    {
+
+      for (j = 0; j < 2; j++)
+        {
+          float current_scale;
+          int offset = i * (j ? 1 : -1);
+
+          scaled_w = base_scaled_w + offset;
+          current_scale = width / scaled_w;
+          scaled_h = height / current_scale;
+
+          if (current_scale >= scale + scale_step ||
+              current_scale <= scale - scale_step ||
+              current_scale < MINIMUM_SCALE_FACTOR ||
+              current_scale > MAXIMUM_SCALE_FACTOR)
+            {
+              limit_exceeded = TRUE;
+              continue;
+            }
+
+          if (floorf (scaled_h) == scaled_h)
+            {
+              found_one = TRUE;
+
+              if (fabsf (current_scale - scale) < fabsf (best_scale - scale))
+                best_scale = current_scale;
+            }
+        }
+
+      i++;
+    }
+  while (!found_one && !limit_exceeded);
+
+out:
+  return best_scale;
+}
+
+float *
+meta_monitor_calculate_supported_scales (MetaMonitor                *monitor,
+                                         MetaMonitorMode            *monitor_mode,
+                                         MetaMonitorScalesConstraint constraints,
+                                         int                        *n_supported_scales)
+{
+  unsigned int i, j;
+  int width, height;
+  float scale_steps;
+  GArray *supported_scales;
+
+  scale_steps = 1.0 / (float) SCALE_FACTORS_PER_INTEGER;
+  supported_scales = g_array_new (FALSE, FALSE, sizeof (float));
+
+  meta_monitor_mode_get_resolution (monitor_mode, &width, &height);
+
+  for (i = floorf (MINIMUM_SCALE_FACTOR);
+       i <= ceilf (MAXIMUM_SCALE_FACTOR);
+       i++)
+    {
+      for (j = 0; j < SCALE_FACTORS_PER_INTEGER; j++)
+        {
+          float scale;
+          float scale_value = i + j * scale_steps;
+
+          if ((constraints & META_MONITOR_SCALES_CONSTRAINT_NO_FRAC) &&
+              fmodf (scale_value, 1.0) != 0.0)
+            {
+              continue;
+            }
+
+          scale = get_closest_scale_factor_for_resolution (width,
+                                                           height,
+                                                           scale_value,
+                                                           scale_steps);
+
+          if (scale > 0.0f)
+            g_array_append_val (supported_scales, scale);
+        }
+    }
+
+  *n_supported_scales = supported_scales->len;
+  return (float *) g_array_free (supported_scales, FALSE);
+}
+
 MetaMonitorModeSpec *
 meta_monitor_mode_get_spec (MetaMonitorMode *monitor_mode)
 {
   return &monitor_mode->spec;
 }
 
+const char *
+meta_monitor_mode_get_id (MetaMonitorMode *monitor_mode)
+{
+  return monitor_mode->id;
+}
+
 void
 meta_monitor_mode_get_resolution (MetaMonitorMode *monitor_mode,
                                   int             *width,
@@ -1357,6 +1559,12 @@ meta_monitor_mode_get_refresh_rate (MetaMonitorMode *monitor_mode)
   return monitor_mode->spec.refresh_rate;
 }
 
+MetaCrtcModeFlag
+meta_monitor_mode_get_flags (MetaMonitorMode *monitor_mode)
+{
+  return monitor_mode->spec.flags;
+}
+
 gboolean
 meta_monitor_mode_foreach_crtc (MetaMonitor        *monitor,
                                 MetaMonitorMode    *mode,
