@@ -24,11 +24,6 @@
 #include "backends/meta-backend-private.h"
 #include "backends/meta-monitor-config-manager.h"
 
-static float supported_scales_test[] = {
-  1.0,
-  2.0
-};
-
 struct _MetaMonitorManagerTest
 {
   MetaMonitorManager parent;
@@ -123,9 +118,16 @@ meta_monitor_manager_test_ensure_initial_config (MetaMonitorManager *manager)
   config = meta_monitor_manager_ensure_configured (manager);
 
   if (meta_is_monitor_config_manager_enabled ())
-    meta_monitor_manager_update_logical_state (manager, config);
+    {
+      meta_monitor_manager_update_logical_state (manager, config);
+    }
   else
-    meta_monitor_manager_update_logical_state_derived (manager);
+    {
+      MetaMonitorManagerDeriveFlag flags =
+        META_MONITOR_MANAGER_DERIVE_FLAG_NONE;
+
+      meta_monitor_manager_update_logical_state_derived (manager, flags);
+    }
 }
 
 static void
@@ -279,9 +281,16 @@ meta_monitor_manager_test_apply_monitors_config (MetaMonitorManager      *manage
       manager->screen_height = 1;
 
       if (meta_is_stage_views_enabled ())
-        meta_monitor_manager_rebuild (manager, NULL);
+        {
+          meta_monitor_manager_rebuild (manager, NULL);
+        }
       else
-        meta_monitor_manager_rebuild_derived (manager);
+        {
+          MetaMonitorManagerDeriveFlag flags =
+            META_MONITOR_MANAGER_DERIVE_FLAG_CONFIGURED_SCALE;
+
+          meta_monitor_manager_rebuild_derived (manager, flags);
+        }
 
       return TRUE;
     }
@@ -311,9 +320,16 @@ meta_monitor_manager_test_apply_monitors_config (MetaMonitorManager      *manage
   update_screen_size (manager, config);
 
   if (meta_is_stage_views_enabled ())
-    meta_monitor_manager_rebuild (manager, config);
+    {
+      meta_monitor_manager_rebuild (manager, config);
+    }
   else
-    meta_monitor_manager_rebuild_derived (manager);
+    {
+      MetaMonitorManagerDeriveFlag flags =
+        META_MONITOR_MANAGER_DERIVE_FLAG_CONFIGURED_SCALE;
+
+      meta_monitor_manager_rebuild_derived (manager, flags);
+    }
 
   return TRUE;
 }
@@ -343,9 +359,12 @@ meta_monitor_manager_test_apply_configuration (MetaMonitorManager *manager,
                                                MetaOutputInfo    **outputs,
                                                unsigned int        n_outputs)
 {
+  MetaMonitorManagerDeriveFlag flags =
+    META_MONITOR_MANAGER_DERIVE_FLAG_NONE;
+
   apply_crtc_assignments (manager, crtcs, n_crtcs, outputs, n_outputs);
   legacy_calculate_screen_size (manager);
-  meta_monitor_manager_rebuild_derived (manager);
+  meta_monitor_manager_rebuild_derived (manager, flags);
 }
 
 static void
@@ -376,7 +395,7 @@ meta_monitor_manager_test_is_transform_handled (MetaMonitorManager  *manager,
   return manager_test->handles_transforms;
 }
 
-static int
+static float
 meta_monitor_manager_test_calculate_monitor_mode_scale (MetaMonitorManager *manager,
                                                         MetaMonitor        *monitor,
                                                         MetaMonitorMode    *monitor_mode)
@@ -390,22 +409,38 @@ meta_monitor_manager_test_calculate_monitor_mode_scale (MetaMonitorManager *mana
   return output_test->scale;
 }
 
-static void
-meta_monitor_manager_test_get_supported_scales (MetaMonitorManager *manager,
-                                                float             **scales,
-                                                int                *n_scales)
+static float *
+meta_monitor_manager_test_calculate_supported_scales (MetaMonitorManager          *manager,
+                                                      MetaLogicalMonitorLayoutMode layout_mode,
+                                                      MetaMonitor                 *monitor,
+                                                      MetaMonitorMode             *monitor_mode,
+                                                      int                         *n_supported_scales)
 {
-  *scales = supported_scales_test;
-  *n_scales = G_N_ELEMENTS (supported_scales_test);
+  MetaMonitorScalesConstraint constraints =
+    META_MONITOR_SCALES_CONSTRAINT_NONE;
+
+  switch (layout_mode)
+    {
+    case META_LOGICAL_MONITOR_LAYOUT_MODE_LOGICAL:
+      break;
+    case META_LOGICAL_MONITOR_LAYOUT_MODE_PHYSICAL:
+      constraints |= META_MONITOR_SCALES_CONSTRAINT_NO_FRAC;
+      break;
+    }
+
+  return meta_monitor_calculate_supported_scales (monitor, monitor_mode,
+                                                  constraints,
+                                                  n_supported_scales);
 }
 
 static gboolean
 is_monitor_framebuffer_scaled (void)
 {
   MetaBackend *backend = meta_get_backend ();
+  MetaSettings *settings = meta_backend_get_settings (backend);
 
-  return meta_backend_is_experimental_feature_enabled (
-    backend,
+  return meta_settings_is_experimental_feature_enabled (
+    settings,
     META_EXPERIMENTAL_FEATURE_SCALE_MONITOR_FRAMEBUFFER);
 }
 
@@ -484,7 +519,7 @@ meta_monitor_manager_test_class_init (MetaMonitorManagerTestClass *klass)
   manager_class->tiled_monitor_removed = meta_monitor_manager_test_tiled_monitor_removed;
   manager_class->is_transform_handled = meta_monitor_manager_test_is_transform_handled;
   manager_class->calculate_monitor_mode_scale = meta_monitor_manager_test_calculate_monitor_mode_scale;
-  manager_class->get_supported_scales = meta_monitor_manager_test_get_supported_scales;
+  manager_class->calculate_supported_scales = meta_monitor_manager_test_calculate_supported_scales;
   manager_class->get_capabilities = meta_monitor_manager_test_get_capabilities;
   manager_class->get_max_screen_size = meta_monitor_manager_test_get_max_screen_size;
   manager_class->get_default_layout_mode = meta_monitor_manager_test_get_default_layout_mode;
